@@ -4,4 +4,4 @@ utils::globalVariables(c("CCIcount", "Cell_type_Ligand", "Cell_type_Receptor",
                          "CellChatDB_human", "CellChatDB_mouse",
                          "KEGG.pathway", "LR_pairs_ConnectomeDB_2020",
                          "Distance", "imagecol", "imagerow", "interactors",
-                         "classification", "significant", "n"))
+                         "classification", "significant", "n","Strength"))
